<?php
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Headers: *");

include_once __DIR__ . '/../../config/db-connection.php';
include_once __DIR__ . '/../../config/https_environment.php';
include_once __DIR__ . '/../../utils/utils.php';

$json = file_get_contents('php://input');
    
$json = json_decode($json,true);

// $json = [
//     'id_user' => '37e44344-f0c5-11ec-a869-00e04c0362fc',
//     'id_unit' => '07678cc9-f15b-11ec-a0db-00e04c0362fc',
//     'id_section' => 'f0b74a11-f221-11ec-9768-00e04c0362fc',
//     'id_content' => 'caba25ad-f233-11ec-9768-00e04c0362fc'
// ];

if(empty($json['id_user'])){
    http_response_code(400);
    die(json_encode([
        'debug' => 'id_user'
    ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
}
$id_user = mysqli_real_escape_string($conn,$json['id_user']);


if(empty($json['id_unit'])){
    http_response_code(400);
    die(json_encode([
        'debug' => 'id_unit'
    ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
}
$id_unit = mysqli_real_escape_string($conn,$json['id_unit']);

if(empty($json['id_section'])){
    http_response_code(400);
    die(json_encode([
        'debug' => 'id_section'
    ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
}
$id_section = mysqli_real_escape_string($conn,$json['id_section']);

$with_answers = null;
if(isset($json['with_answers'])){
    $with_answers = mysqli_real_escape_string($conn,$json['with_answers']);
}


// GET CONTENT
$id_content = null;
if(isset($json['id_content'])){
    $id_content = mysqli_real_escape_string($conn,$json['id_content']);

    // GET CONTENT WITH SPECIFIC ID
    $sql_content = "SELECT 
    c.id_content,
    c.type,
    c.order,
    c.title,
    c.meta,
    s.label as section_label, 
    u.label as unit_label,
    l.label as level_label,
    l.class_name,
    l.order as level_order,
    l.id_level,
    u.order as unit_order,
    s.order as section_order
    FROM contents c
    INNER JOIN sections s USING(id_section)
    INNER JOIN units u USING(id_unit)
    INNER JOIN levels l USING(id_level)
    INNER JOIN users us ON us.id_user = '{$id_user}'
    INNER JOIN user_progress up ON up.id_user = us.id_user
    WHERE 
    (c.id_content = '{$id_content}' AND u.id_unit = '{$id_unit}' AND s.id_section = '{$id_section}') AND
    (
        (l.order < up.level_order) OR
        (l.order = up.level_order AND u.order < up.unit_order) OR
        (l.order = up.level_order AND u.order = up.unit_order AND s.order < up.section_order) OR
        (l.order = up.level_order AND u.order = up.unit_order AND s.order = up.section_order AND c.order <= up.content_order )
    ) AND (
        l.status = 1 AND
        u.status = 1 AND
        s.status = 1 AND
        c.status = 1
    )
    ";
}else{
    // GET FIST CONTENT 
    $sql_content = "SELECT 
    c.id_content,
    c.type,
    c.order,
    c.title,
    c.meta,
    s.label as section_label, 
    u.label as unit_label,
    l.label as level_label,
    l.class_name,
    l.order as level_order,
    l.id_level,
    u.order as unit_order,
    s.order as section_order
    FROM contents c
    INNER JOIN sections s USING(id_section)
    INNER JOIN units u USING(id_unit)
    INNER JOIN levels l USING(id_level)
    INNER JOIN users us ON us.id_user = '{$id_user}'
    INNER JOIN user_progress up ON up.id_user = us.id_user
    WHERE (u.id_unit = '{$id_unit}' AND s.id_section = '{$id_section}')
    AND(
        (
            (
                (l.order < up.level_order) OR
                (l.order = up.level_order AND u.order < up.unit_order) OR
                (l.order = up.level_order AND u.order = up.unit_order AND s.order < up.section_order)
            ) AND c.order = 1
        ) OR (
            (l.order = up.level_order AND u.order = up.unit_order AND s.order = up.section_order) AND
            c.order = up.content_order
        )
    )";
}
$result_content = mysqli_query($conn,$sql_content);

if(mysqli_num_rows($result_content) < 1){
    http_response_code(403);
    die();
}

$row_content = mysqli_fetch_assoc($result_content);
$id_content = $row_content['id_content'];
$id_level = $row_content['id_level'];
// END GET CONTENT

// GET ANSWERS
$answer_data = null;
if($with_answers != null){
    
    $sql_answer = "SELECT attempts, num_attempts, correct, created_at FROM answers 
    WHERE id_user = '{$id_user}' AND 
    id_content = '{$id_content}' AND 
    `status` = 1";

    $result_answer = mysqli_query($conn,$sql_answer);

    if(!$result_answer){
        http_response_code(200);
        mysqli_commit($conn);
        mysqli_close($conn);
        die(json_encode([
            'debug' => 'get answer query error'
        ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
    }

    if(mysqli_num_rows($result_answer) > 0){

        $row_answer = mysqli_fetch_assoc($result_answer);
    
        $answer_data = [
            'attempts'      => json_decode($row_answer['attempts'], JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES),
            'num_attempts'  => $row_answer['num_attempts'],
            'correct'       => $row_answer['correct'],
            'created_at'    => $row_answer['created_at']
        ];

        if($row_answer['correct'] != null){
            // GET NEXT CONTENT ORDER
            $content_order = $row_content['order'];
            $level_order = $row_content['level_order'];
            $unit_order = $row_content['unit_order'];
            $section_order = $row_content['section_order'];

            $next_content_data = get_next_content_id($conn,[
                'content' => [ 'id' => $id_content, 'order' => $content_order ],
                'section' => [ 'id' => $id_section, 'order' => $section_order ],
                'unit' => [ 'id' => $id_unit, 'order' => $unit_order ],
                'level' => [ 'id' => $id_level, 'order' => $level_order ]
            ]);

            if($next_content_data['status'] != 200){
                mysqli_rollback($conn);
                mysqli_close($conn);
                http_response_code($next_content_data['status']);
                die(json_encode([
                    'debug' => $next_content_data['debug']
                ], JSON_UNESCAPED_UNICODE, JSON_UNESCAPED_SLASHES));
            }

            $response_next_content = $next_content_data['response'];

            $att_key = $response_next_content['att_key'];
            $next_content_id = $response_next_content['content']['id'];
            
            $answer_data['next_content_data'] = [
                'att_key' => $att_key,
                'next_content' => $next_content_id
            ];

        }

    }
    

}
//

$meta = json_decode($row_content['meta'], true);

unset($meta['answers']);

$json_content['content'] = [
    'id_content'    => $row_content['id_content'],
    'type'          => $row_content['type'],
    'title'         => $row_content['title'],
    'order'         => $row_content['order'],
    'meta'          => $meta,
    'section_label' => $row_content['section_label'],
    'unit_label'    => $row_content['unit_label'],
    'level_label'   => $row_content['level_label'],
    'level_class'   => $row_content['class_name'],
    'answer_data'   => $answer_data
];

http_response_code(200);
die(json_encode($json_content,JSON_UNESCAPED_UNICODE, JSON_UNESCAPED_SLASHES));

    // echo '
    // {
        // "content":{
        //     "contentId":"1",
        //     "contentType":"1",
        //     "order":3,
        //     "title":"Complete the names of the professions:",
        //     "htmlBody":"<div><img width=\\"500\\" src=\\"/uploads/questions/teacher.jpg\\"> </div>",
        //     "contentData":{
        //         "answers":[
        //             "Cab Driver",
        //             "Engineer",
        //             "Pilot",
        //             "At Home",
        //             "Tennis Player",
        //             "Doctor"
        //         ]
        //     }
        // },
        // "section":{
        //     "sectionName": "Book"
        // },
        // "unit":{
        //     "levelName":"Foundation",
        //     "unitName": "Unit 1",
        //     "colorClass":"teal"
        // }
    // }';

?>

