<?php
    header("Access-Control-Allow-Origin: *");
    header("Access-Control-Allow-Headers: *");

    include_once __DIR__ . '/../../config/db-connection.php';
    include_once __DIR__ . '/../../config/https_environment.php';
    include_once __DIR__ . '/../../utils/utils.php';

    $json = file_get_contents('php://input');
    
    $json = json_decode($json,true);

    if(empty($json['id_user'])){
        http_response_code(400);
        mysqli_close($conn);
        die(json_encode([
            'debug' => 'id_user'
        ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
    }
    $id_user = mysqli_real_escape_string($conn,$json['id_user']);

    if(empty($json['answer'])){
        http_response_code(400);
        mysqli_close($conn);
        die(json_encode([
            'debug' => 'answer'
        ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
    }
    $answer = mysqli_real_escape_string($conn,$json['answer']);

    if(empty($json['id_content'])){
        http_response_code(400);
        mysqli_close($conn);
        die(json_encode([
            'debug' => 'id_content'
        ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
    }
    $id_content = mysqli_real_escape_string($conn,$json['id_content']);

//GET CONTENT DATA
    $sql_content = "SELECT c.`type`,
    c.`order`,
    c.`id_section`,
    c.`meta`,
    l.order as level_order,
    l.id_level,
    u.order as unit_order,
    u.id_unit,
    s.order as section_order
    FROM contents c
    INNER JOIN sections s ON c.id_section = s.id_section
    INNER JOIN units u ON u.id_unit = s.id_unit
    INNER JOIN `levels` l ON u.id_level = l.id_level
    WHERE c.id_content = '${id_content}' AND
    c.status = 1";
    $result_content = mysqli_query($conn,$sql_content);
    if(!$result_content){
        http_response_code(500);
        die(json_encode([
            'debug' => 'erro na query, ' . $sql_content
        ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
    }

    if(mysqli_num_rows($result_content) < 1){
        http_response_code(403);
        die(json_encode([
            'debug' => 'content not found'
        ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
    }
    $row_content = mysqli_fetch_assoc($result_content);
    $id_section = $row_content['id_section'];
    $id_unit = $row_content['id_unit'];
    $id_level = $row_content['id_level'];
    $type_content = $row_content['type'];
    $content_order = $row_content['order'];
    $meta_content = json_decode($row_content['meta'],true);
    $correct_answers = $meta_content['answers'];
// GET CONTENT PERMITIONS
    $level_order = $row_content['level_order'];
    $unit_order = $row_content['unit_order'];
    $section_order = $row_content['section_order'];
// GET USER PROGRESS PERMITION
    $result_user_progress = get_user_progress($id_user,$url);

    if($result_user_progress['status'] != 200){
        http_response_code($result_user_progress['status']);
        die(json_encode([
            'debug' => $result_user_progress['debug']
        ], JSON_UNESCAPED_UNICODE, JSON_UNESCAPED_SLASHES));
    }

    // $user_progress_json = json_decode($user_progress, true);
    $user_progress_json = $result_user_progress['response']['user_progress'];
    $user_unit_progress = json_decode($user_progress_json['unitProgress'],true);
    $user_progress = $user_progress_json['progress'];

// TEST USER PROGRESS PERMITION
    $user_progress_authorized = check_user_permission_material([
        'level' => $level_order,
        'unit' => $unit_order,
        'section' => $section_order,
        'content' => $content_order
    ],$user_progress);

    if(!$user_progress_authorized){
        http_response_code(403);
        mysqli_close($conn);
        die(json_encode([
            'debug' => 'user progress 403'
        ], JSON_UNESCAPED_UNICODE, JSON_UNESCAPED_SLASHES));
    }

// GET LATEST ANSWERS AND VERIFY
    $sql_last_answer = "SELECT num_attempts, attempts, correct FROM answers WHERE id_user = '{$id_user}' AND id_content = '{$id_content}'";
    $result_last_answer = mysqli_query($conn,$sql_last_answer);
    if(!$result_last_answer){
        http_response_code(500);
        var_dump(mysqli_error($conn));
        mysqli_close($conn);
        die(json_encode([
            'debug' => 'erro na query, ' . $sql_last_answer
        ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
    }
    $attemps = [];
    $num_attempts = 1;
    $new_answer = true;
    if(mysqli_num_rows($result_last_answer) > 0){
        $new_answer = false;
        $row_last_answer = mysqli_fetch_assoc($result_last_answer);
        $correct = $row_last_answer['correct'];
        $num_attempts = intval($row_last_answer['num_attempts']);
        $num_attempts++;
        $attemps = json_decode($row_last_answer['attempts'],true);

        // CHECK ATTEMPTS
        if($correct != null || $num_attempts > intval($meta_content['contentData']['attempts'])){
            http_response_code(403);
            mysqli_close($conn);
            die();
        }
    }
    
// SEND ANSWER BY CONTENT TYPE
    mysqli_begin_transaction($conn);
    $answer_correct = 'null';
    switch($type_content){
        case 1:
            // VERIFY OPTIONS
            $options = $meta_content['contentData']['options'];
            if($options[$answer] == null){
                http_response_code(400);
                mysqli_close($conn);
                die(json_encode([
                    'debug' => 'option não encontrada, ' . $answer
                ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
            }

            // CHECK ANSWER
            if($answer == $correct_answers){
                $answer_correct = 1;
            }else if($num_attempts >= intval($meta_content['contentData']['attempts'])){
                $answer_correct = 0;
            }

            // REGISTER ATTEMPT
            $attemps[$answer] = $answer == $correct_answers ? true : false;
            $json_attempts = json_encode($attemps, JSON_UNESCAPED_UNICODE, JSON_UNESCAPED_SLASHES);

            if($new_answer){
                $sql_push_answer = "CALL sp_create_answer(
                    '{$id_user}',
                    '{$id_content}',
                    '${json_attempts}',
                    {$answer_correct}
                )";
            }else{
                $sql_push_answer = "CALL sp_update_answer(
                    '{$id_user}',
                    '{$id_content}',
                    '{$json_attempts}',
                    {$num_attempts},
                    {$answer_correct}
                )";
            }
            
            $result_push = mysqli_query($conn,$sql_push_answer);

            if(!$result_push){
                mysqli_rollback($conn);
                mysqli_close($conn);
                http_response_code(500);
                die(json_encode([
                    'debug' => 'erro na query, ' . $sql_push_answer
                ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
            }

            break;
        default:
            http_response_code(500);
            die(json_encode([
                'debug' => 'Tipo não reconhecido: ' . $type_content
            ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
            break;
    }
    
    mysqli_next_result($conn);

// GET NEXT CONTENT ID / REGISTER NEW PROGRESS ( if necessary )
    if($answer_correct == 1 || $num_attempts >= intval($meta_content['contentData']['attempts'])){
    // GET NEXT CONTENT ORDER
        $next_content_data = get_next_content_id($conn,[
            'content' => [ 'id' => $id_content, 'order' => $content_order ],
            'section' => [ 'id' => $id_section, 'order' => $section_order ],
            'unit'    => [ 'id' => $id_unit, 'order' => $unit_order ],
            'level'   => [ 'id' => $id_level, 'order' => $level_order ]
        ]);

        if($next_content_data['status'] != 200){
            mysqli_rollback($conn);
            mysqli_close($conn);
            http_response_code($next_content_data['status']);
            die(json_encode([
                'debug' => $next_content_data['debug']
            ], JSON_UNESCAPED_UNICODE, JSON_UNESCAPED_SLASHES));
        }

        $response_next_content =    $next_content_data['response'];
        $att_key =                  $response_next_content['att_key'];

        if($att_key != 'end'){
            $next_level_order =         isset($response_next_content['level']) ?      intval($response_next_content['level']['order']) : 0;
            $next_unit_order =          isset($response_next_content['unit']) ?       intval($response_next_content['unit']['order']) : 0;
            $next_section_order =       isset($response_next_content['section']) ?    intval($response_next_content['section']['order']) : 0;
            $next_content_order =       isset($response_next_content['content']) ?    intval($response_next_content['content']['order']) : 0;
    
        // VERIFY IF NEXT CONTENT ARE HIGHER THAN ACTUAL USER PROGRESS
            $update_user_progress = false;
            $quit = false;
            if($next_level_order > intval($user_progress['level'])){
                $user_progress['level'] = $next_level_order;
                $user_progress['unit'] = $next_unit_order;
                $user_progress['section'] = $next_section_order;
                $user_progress['content'] = $next_content_order;
                $update_user_progress = true;
                $quit = true;
    
            }else if($next_level_order == intval($user_progress['level']) && $next_unit_order > intval($user_progress['unit'])){
                $user_progress['unit'] = $next_unit_order;
                $user_progress['section'] = $next_section_order;
                $user_progress['content'] = $next_content_order;
                $update_user_progress = true;
                $quit = true;
    
            }else if($next_level_order == intval($user_progress['level']) && $next_unit_order == intval($user_progress['unit']) && $next_section_order > intval($user_progress['section'])){
                $user_progress['section'] = $next_section_order;
                $user_progress['content'] = $next_content_order;
                $update_user_progress = true;
                $quit = true;
    
            }else if($next_level_order == intval($user_progress['level']) && $next_unit_order == intval($user_progress['unit']) && $next_section_order == intval($user_progress['section']) && $next_content_order > intval($user_progress['content'])){
                $user_progress['content'] = $next_content_order;
                $update_user_progress = true;
            }
        }else{
            $update_user_progress = false;
            $quit = true;
        }


    // UPDATE GRADE
        $sql_grade = "SELECT SUM(a.correct) as points, COUNT(*) as total 
        FROM answers a
        INNER JOIN contents c USING (id_content)
        INNER JOIN sections s USING (id_section)
        INNER JOIN units u USING (id_unit)
        WHERE NOT ISNULL(a.correct) AND
        u.id_unit = '{$id_unit}' AND
        a.id_user = '{$id_user}' AND
        c.status = 1";

        $result_grade = mysqli_query($conn,$sql_grade);

        if(!$result_grade){
            $error = mysqli_error($conn);
            mysqli_rollback($conn);
            mysqli_close($conn);
            http_response_code(500);
            die(json_encode([
                'debug' => $error
            ], JSON_UNESCAPED_UNICODE, JSON_UNESCAPED_SLASHES));
        }

        $row_grade = mysqli_fetch_assoc($result_grade);

        $answers_points = $row_grade['points'];
        $total_answers = $row_grade['total'];
        $grade = intval(($answers_points * 100)/ $total_answers);

        if($user_unit_progress == null){
            $user_unit_progress = [];
        }

        $user_unit_progress[$id_unit]['grade'] = $grade;
    
    // UPDATE UNIT PROGRESS
        $sql_total_unit = "SELECT count(*) as total
        FROM contents c
        INNER JOIN sections s USING(id_section)
        INNER JOIN units u USING(id_unit)
        WHERE u.id_unit = '{$id_unit}' AND
        c.status = 1;";

        $result_total_unit = mysqli_query($conn,$sql_total_unit);

        if(!$result_total_unit){
            mysqli_rollback($conn);
            mysqli_close($conn);
            http_response_code(500);
            die(json_encode([
                'debug' => $sql_total_unit
            ], JSON_UNESCAPED_UNICODE, JSON_UNESCAPED_SLASHES));
        }

        $row_total_unit = mysqli_fetch_assoc($result_total_unit);
        $total_unit = $row_total_unit['total'];

        $unit_progress = intval(($total_answers * 100)/ $total_unit);

        if($user_unit_progress == null){
            $user_unit_progress = [];
        }

        $user_unit_progress[$id_unit]['progress'] = $unit_progress;
        
        $json_unit_progress = json_encode($user_unit_progress,true);
    // SUBMIT UPDATE
        $sql_update_progress = "CALL sp_update_progress(
            '{$id_user}',
            '{$id_user}',
            '{$user_progress['level']}',
            '{$user_progress['unit']}',
            '{$user_progress['section']}',
            '{$user_progress['content']}',
            '{$json_unit_progress}'
        )";

        $result_update_progress = mysqli_query($conn,$sql_update_progress);

        if(!$result_update_progress){
            mysqli_rollback($conn);
            mysqli_close($conn);
            http_response_code(500);
            die(json_encode([
                'debug' => $sql_update_progress
            ], JSON_UNESCAPED_UNICODE, JSON_UNESCAPED_SLASHES));
        }
        mysqli_next_result($conn);
    //
    
        http_response_code(200);
        mysqli_commit($conn);
        mysqli_close($conn);
        die(json_encode([
            'answer_correct' => $answer_correct,
            'update_user_progress' => $update_user_progress,
            'user_progress' => $user_progress,
            'next_content_data' => [
                'quit' => $quit,
                'att_key' => $att_key,
                'next_content' => $response_next_content
            ] 
        ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));

    }

//END OPERATION
    http_response_code(200);
    mysqli_commit($conn);
    mysqli_close($conn);
    die(json_encode([
        'answer_correct' => false,
        'next_content_data' => false
    ],JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
?>
