<?php

function json_curl($url_curl,$json){
    // Init curl
    $ch = curl_init($url_curl);
    // Setup request to send json via POST.
    $payload = json_encode($json);
    curl_setopt( $ch, CURLOPT_POSTFIELDS, $payload );
    curl_setopt( $ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
    // Return response instead of printing.
    curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
    // Send request.
    $result = curl_exec($ch);
    curl_close($ch);
    
    $return = [
        'status' =>  curl_getinfo($ch, CURLINFO_HTTP_CODE),
        'response' => $result
    ];

    return json_encode($return, JSON_UNESCAPED_UNICODE, JSON_UNESCAPED_SLASHES);
}

/**
 * @param json $content_permitions Os valores int da ordem dos levels,units,sections e contents. Levels é requisitado, units,sections e contents são opcionais
 * @param json $user_permitions Os valores int da ordem dos levels,units,sections e contents do progresso do usuário. Todos os valores são requisitados para a integridade do teste.
 * 
 * @return BOOL
 */
function check_user_permission_material($content_permitions,$user_permitions){

    if(empty($content_permitions['level'])){
        http_response_code(400);
        die();
    }

    $level_order = $content_permitions['level'];
    $unit_order = isset($content_permitions['unit']) ? $content_permitions['unit'] : 0;
    $section_order = isset($content_permitions['section']) ? $content_permitions['section'] : 0;
    $content_order = isset($content_permitions['content']) ? $content_permitions['content'] : 0;

    if(
        ($level_order < $user_permitions['level']) ||
        ($level_order == $user_permitions['level'] && $unit_order < $user_permitions['unit']) ||
        ($level_order == $user_permitions['level'] && $unit_order == $user_permitions['unit'] && $section_order < $user_permitions['section']) ||
        ($level_order == $user_permitions['level'] && $unit_order == $user_permitions['unit'] && $section_order == $user_permitions['section'] && $content_order <= $user_permitions['content'] )
    ){
        return true;
    }else{
        return false;
    }
}

function get_next_content_id($conn,$json){
    // NEXT CONTENT
        $sql_content = "SELECT
            c.`id_content`,
            c.`order`,

            s.id_section,
            s.order as order_section,

            u.id_unit,
            u.order as order_unit,

            l.id_level,
            l.order as order_level

            FROM contents c
            INNER JOIN sections s USING(id_section)
            INNER JOIN units u USING(id_unit)
            INNER JOIN levels l USING(id_level)
            WHERE c.`order` > {$json['content']['order']} AND 
            c.`id_section` = '{$json['section']['id']}' AND 

            c.`status` = 1 AND
            s.`status` = 1 AND
            u.`status` = 1 AND
            l.`status` = 1 

            ORDER BY c.`order`
            LIMIT 1;
        ";
        
        $result_content = mysqli_query($conn,$sql_content);

        if(!$result_content){
            return [
                'status' => 500,
                'debug' => $sql_content
            ];
        }

        if(mysqli_num_rows($result_content) > 0){
            $row = mysqli_fetch_assoc($result_content);

            $id_content = $row['id_content'];
            $order_content = $row['order'];

            $id_section = $row['id_section'];
            $order_section = $row['order_section'];

            $id_unit = $row['id_unit'];
            $order_unit = $row['order_unit'];

            $id_level = $row['id_level'];
            $order_level = $row['order_level'];
            
            return [
                'status' => 200,
                'response' => [
                    'att_key' => 'content',
                    'content' => [
                        'id'    => $id_content,
                        'order' => $order_content
                    ],
                    'section' => [
                        'id'    => $id_section,
                        'order' => $order_section
                    ],
                    'unit' => [
                        'id'    => $id_unit,
                        'order' => $order_unit
                    ],
                    'level' => [
                        'id'    => $id_level,
                        'order' => $order_level
                    ],
                ]
            ];
        }

    // NEXT SECTION
        $sql_section = "SELECT
            c.`id_content`,
            c.`order` as order_content,
            s.id_section,
            s.order as order_section,

            u.id_unit,
            u.order as order_unit,

            l.id_level,
            l.order as order_level
            FROM contents c
            INNER JOIN sections s USING(id_section)
            INNER JOIN units u USING(id_unit)
            INNER JOIN levels l USING(id_level)
            WHERE s.`order` > {$json['section']['order']} AND
            s.id_unit = '{$json['unit']['id']}' AND

            c.`status` = 1 AND
            s.`status` = 1 AND
            u.`status` = 1 AND
            l.`status` = 1 

            ORDER BY s.`order` ASC, c.`order` ASC
            LIMIT 1;
        ";

        $result_section = mysqli_query($conn,$sql_section);

        if(!$result_section){
            return [
                'status' => 500,
                'debug' => $sql_section
            ];
        }

        if(mysqli_num_rows($result_section) > 0){
            $row = mysqli_fetch_assoc($result_section);

            $id_content = $row['id_content'];
            $order_content = $row['order_content'];

            $id_section = $row['id_section'];
            $order_section = $row['order_section'];

            $id_unit = $row['id_unit'];
            $order_unit = $row['order_unit'];

            $id_level = $row['id_level'];
            $order_level = $row['order_level'];
            
            return [
                'status' => 200,
                'response' => [
                    'att_key' => 'section',
                    'content' => [
                        'id'    => $id_content,
                        'order' => $order_content
                    ],
                    'section' => [
                        'id'    => $id_section,
                        'order' => $order_section
                    ],
                    'unit' => [
                        'id'    => $id_unit,
                        'order' => $order_unit
                    ],
                    'level' => [
                        'id'    => $id_level,
                        'order' => $order_level
                    ],
                ]
            ];
        }
    // NEXT UNIT
        $sql_unit = "SELECT
            c.`id_content`,
            c.`order` as order_content,

            s.id_section,
            s.order as order_section,

            u.id_unit,
            u.order as order_unit,

            l.id_level,
            l.order as order_level

            FROM contents c
            INNER JOIN sections s USING(id_section)
            INNER JOIN units u USING(id_unit)
            INNER JOIN levels l USING(id_level)
            WHERE u.`order` > {$json['unit']['order']} AND
            u.id_level = '{$json['level']['id']}' AND

            c.`status` = 1 AND
            s.`status` = 1 AND
            u.`status` = 1 AND
            l.`status` = 1 

            ORDER BY u.`order` ASC, s.`order` ASC, c.`order` ASC
            LIMIT 1;
        ";

        $result_unit = mysqli_query($conn,$sql_unit);

        if(!$result_unit){
            return [
                'status' => 500,
                'debug' => $sql_unit
            ];
        }

        if(mysqli_num_rows($result_unit) > 0){
            $row = mysqli_fetch_assoc($result_unit);

            $id_content = $row['id_content'];
            $order_content = $row['order_content'];

            $id_section = $row['id_section'];
            $order_section = $row['order_section'];

            $id_unit = $row['id_unit'];
            $order_unit = $row['order_unit'];

            $id_level = $row['id_level'];
            $order_level = $row['order_level'];
            
            return [
                'status' => 200,
                'response' => [
                    'att_key' => 'unit',
                    'content' => [
                        'id'    => $id_content,
                        'order' => $order_content
                    ],
                    'section' => [
                        'id'    => $id_section,
                        'order' => $order_section
                    ],
                    'unit' => [
                        'id'    => $id_unit,
                        'order' => $order_unit
                    ],
                    'level' => [
                        'id'    => $id_level,
                        'order' => $order_level
                    ],
                ]
            ];
        }
    // NEXT LEVEL
        $sql_level = "SELECT
            c.`id_content`,
            c.`order` as order_content,

            s.id_section,
            s.order as order_section,

            u.id_unit,
            u.order as order_unit,

            l.id_level,
            l.order as order_level

            FROM contents c
            INNER JOIN sections s ON c.id_section = s.id_section
            INNER JOIN units u ON s.id_unit = u.id_unit
            INNER JOIN levels l ON l.id_level = u.id_level
            WHERE l.`order` > {$json['level']['order']} AND

            c.`status` = 1 AND
            s.`status` = 1 AND
            u.`status` = 1 AND
            l.`status` = 1 

            ORDER BY l.`order` ASC, u.`order` ASC, s.`order` ASC, c.`order` ASC
            LIMIT 1;
        ";

        $result_level = mysqli_query($conn,$sql_level);

        if(!$result_level){
            return [
                'status' => 500,
                'debug' => $sql_level
            ];
        }

        if(mysqli_num_rows($result_level) > 0){
            $row = mysqli_fetch_assoc($result_level);

            $id_content = $row['id_content'];
            $order_content = $row['order'];

            $id_section = $row['id_section'];
            $order_section = $row['order_section'];

            $id_unit = $row['id_unit'];
            $order_unit = $row['order_unit'];

            $id_level = $row['id_level'];
            $order_level = $row['order_level'];
            
            return [
                'status' => 200,
                'response' => [
                    'att_key' => 'level',
                    'content' => [
                        'id'    => $id_content,
                        'order' => $order_content
                    ],
                    'section' => [
                        'id'    => $id_section,
                        'order' => $order_section
                    ],
                    'unit' => [
                        'id'    => $id_unit,
                        'order' => $order_unit
                    ],
                    'level' => [
                        'id'    => $id_level,
                        'order' => $order_level
                    ],
                ]
            ];
        }else{
            return [
                'status' => 200,
                'response' => [
                    'att_key' => 'end',
                    'content' => [
                        'id' => 'end'
                    ]
                ]
            ];
        }
}

function get_user_progress($id_user,$url){
    // GET USER PROGRESS PERMITION
    $result_user_progress = json_curl(
        "{$url}/usuarios/progressao/buscar-progresso.php",
        ['id_user' => $id_user]
    );
    try{
        $return_user_progress = json_decode($result_user_progress, true);
        if(gettype($return_user_progress) != 'array'){
            return [
                'status' => 500,
                'debug' => 'curl not array',
                'response' => []
            ];
        }

        if($return_user_progress['status'] != 200){
            return [
                'status' => $return_user_progress['status'],
                'debug' => 'curl status failed',
                'response' => []
            ];
        }

        $user_progress = json_decode($return_user_progress['response'], true);

        return [
            'status' => 200,
            'response' => [
                'user_progress' => $user_progress
            ]
        ];

    }catch(Exception $e){
        return [
            'status' => 500,
            'debug' => $e,
            'response' => []
        ];
    }
}

